//     Author: Maurizio Loreti, aka MLO or (HAM) I3NOO
//     Work:   University of Padova - Department of Physics
//             Via F. Marzolo, 8 - 35131 PADOVA - Italy
//     Phone:  +39 (049) 827-7216   FAX: +39 (049) 827-7102
//     EMail:  loreti@pd.infn.it
//     WWW:    http://www.pd.infn.it/~loreti/mlo.html
//
// -------------------------------------------------------------
//
// $Id: table.cxx,v 1.3 2005/01/21 07:15:43 loreti Exp $
//
// Some procedures defined in table.hh
//
// -------------------------------------------------------------

#include <iostream>
#include <iomanip>
#include "table.hh"

using namespace std;

bool cardStack::operator > (
  const cardStack &rhs
) const {

// Looks if the first card in "this" is bigger than the first card in
// "rhs"; i.e. if the suit value is bigger, or the suit is the same
// and the value is bigger.

  const card &c1(head->theCard);
  const card &c2(rhs.head->theCard);

  int s1, s2;
  s1 = c1.getSuit();
  s2 = c2.getSuit();
  if (s1 > s2) return true;
  if (s1 < s2) return false;

  int v1, v2;
  v1 = c1.getValue();
  v2 = c2.getValue();
  return v1 > v2;
}

void cardStack::add (
  const card &newCard
) {

// Adds "newCard" at the beginning of the linked list

  head = new cardNode(newCard, 0, head);
  howMany++;
}

const card & cardStack::remove (
) {

// Removes a cardNode from the beginning of the linked list; returns a
// constant reference to the removed card. For "level", see the
// comments in table.hh and solve.cxx .

  assert(head != 0);

  cardNode *pN = head;
  head = pN->next;
  const card &retval(pN->theCard);
  delete pN;
  howMany--;
  if (--level < 0) {            // Moved out a 10?
    level = howMany - 1;        // If the next card is another 10,
  }                             // level will be modified from moveIn().
  return retval;
}

void cardStack::removeAll (
) {

// Deletes all the cardNodes in the linked list

  cardNode *pN = head;
  while (pN != 0) {
    cardNode *child = pN->next;
    delete pN;
    pN = child;
  }
  head = 0;
  howMany = 0;
}

void cardStack::print(
  ostream &o
) const {
  cardNode *pN = head;

  o << " [ ";
  while (pN != 0) {
    o << pN->theCard;
    pN=pN->next;
  }
  o << " ]";
}

table::table (
  const pack &p
) : firstStack(0) {
  int i;
  vector<card>::const_iterator ci( p.getCI() );

  for (i = 0;   i < 4;   i++) {
    lastValue[i] = 0;
  }

  for (i = 0;   i < 16;   i++) {
    pStack[i] = new cardStack;
    int j = (i < 8 ? 2 : 1);
    pStack[i]->level = j;
    do {
      pStack[i]->add( *ci++ );
    } while (j--);
  }
}

table::~table()
{
  for (int i = 0;   i < 16;   i++) {
    delete pStack[i];
  }
}

void table::print (
  ostream &o
) const {
  int i;

  o << "--- Foundations:";
  for (i = 0;   i < 4;   i++) {
    o << ' ' << card::suitName[i] << card::valueName[ lastValue[i] ];
  }
  if (firstStack) {
    o << "\n    Stacks 0-" << firstStack-1 << " : empty";
  }
  for (i = firstStack;   i < 16;   i++) {
    o << "\n    Stack " << setw(2) << i << " (level "
      << pStack[i]->level << ") : " << *(pStack[i]);
  }
}
